#version 330

// https://en.wikibooks.org/wiki/GLSL_Programming/GLUT/Multiple_Lights
struct PointLight
{
	vec3 Position;
	vec4 diffuse;
	vec4 specular;
	float constantAttenuation, linearAttenuation, quadraticAttenuation;
	float spotCutoff, spotExponent;
	vec3 spotDirection;	
};

uniform sampler2D colorTexture;

uniform mat4 viewMatrix;
uniform int lightPointsLen; 
//uniform vec3 lightPosition_worldspace;
uniform PointLight pointLights[6]; 

in vec2 UVs;
in vec3 position_worldspace; // position of the vertex (and fragment) in world space
in vec3 normal_cameraspace;
in vec3 eyeDirection_cameraspace; // view direction
in vec3 varyingNormalDirection; 
out vec4 fragColor;

vec4 scene_ambient = vec4(0.2, 0.2, 0.2, 1.0);

void main()
{

	vec3 color = vec3(1,1,1);
	float lightIntensity = 30.0f;
	//-2 is the bias. You’ll have to experiment with this value. The screenshot above was 
	//taken with a bias of +2, which means that OpenGL will select two mipmaps
	//above the one it should have taken (so it’s 16 times smaller, hence the glitches)
	float sharpness = -2;
	vec3 MaterialDiffuseColor = texture( colorTexture, UVs, sharpness).rgb;
	vec3 MaterialAmbientColor = vec3(0.3,0.3,0.3) * MaterialDiffuseColor;
	vec3 MaterialSpecularColor = MaterialDiffuseColor * vec3(0.5,0.5,0.5);
	
	vec3 normalDirection = normalize(varyingNormalDirection);
	vec3 viewDirection = normalize(vec3(inverse(viewMatrix) * vec4(0.0, 0.0, 0.0, 1.0) - vec4(position_worldspace, 1.0)));
	vec3 totalLighting = vec3(scene_ambient) * MaterialDiffuseColor;
	
	vec3 n = normalize(normal_cameraspace); // Normal of the computed fragment, in camera space, normal direction
	vec3 E = normalize(eyeDirection_cameraspace); 	// Eye vector (towards the camera)
	
	float cosTheta;
	float cosAlpha; 
	float distance; 
	vec3 lightDirection; 
	float attenuation;
	//n = vec3(1, 1, 1);
	// TODO improve function
	for (int i=0; i < lightPointsLen; i++){
		/*distance = length( pointLights[i].Position - position_worldspace ); // distance to light
		// Vector that goes from the vertex to the light, in camera space. M is ommited because it's identity
		//lightDirection = ( viewMatrix * vec4(pointLights[i].Position,1)).xyz  + eyeDirection_cameraspace;
		
		attenuation = 1.0 / (pointLights[i].constantAttenuation + (pointLights[i].linearAttenuation 
			* distance) + (pointLights[i].quadraticAttenuation * distance * distance));
		//lightDirection = (pointLights[i].Position); // for directionallight TODO
		lightDirection = vec3(pointLights[i].Position - position_worldspace);
		vec3 l = normalize( lightDirection );  // Direction of the light (from the fragment to the light), lightDirection
		
		 if (pointLights[i].spotCutoff <= 90.0){
			   // spotlight?
			  float clampedCosine = max(0.0, dot(-l, normalize(pointLights[i].spotDirection)));
			  if (clampedCosine < cos(radians(pointLights[i].spotCutoff))){
				// outside of spotlight cone?
				attenuation = 0.0;
			}
			 else{
			  attenuation = attenuation * pow(clampedCosine, pointLights[i].spotExponent);   
			}
		}
		
		// calculate diffuse color
		// TODO: diffuse color of vector 
		vec3 diffuseReflection = clamp(attenuation 
		* vec3(pointLights[i].diffuse) * MaterialDiffuseColor 
		* max(0.0, dot(n, l)), 0.0, 1.0);
		// TODO: set specular, 
		vec3 specularReflection; 
		if (dot(n, l) < 0.0){
			// light source on the wrong side?
			specularReflection = vec3(0.0, 0.0, 0.0); // no specular reflection
		}else{
			 // light source on the right side
			 // setSpecular
			specularReflection	= attenuation * vec3(pointLights[i].specular) * MaterialSpecularColor 
				* pow(max(0.0, dot(reflect(-l, n), E)), 1.0); 
		}

		totalLighting += diffuseReflection + specularReflection;*/

	}
	
	fragColor.rgb += totalLighting; 
	fragColor.a=1;




		
}

   