#version 330

// variante 1
uniform sampler2D diffuseTexture;
uniform sampler2DShadow shadowMap;


struct PointLight
{
	vec3 Position;
	vec4 diffuse;
	vec4 specular;
	float constantAttenuation, linearAttenuation, quadraticAttenuation;
	float spotCutoff, spotExponent;
	vec3 spotDirection;	
	float intensity; 
};

uniform sampler2D colorTexture;

uniform mat4 viewMatrix;
uniform int lightPointsLen; 
//uniform vec3 lightPosition_worldspace;
uniform PointLight pointLights[15]; 
uniform float transparency; 
uniform int activeLight; 
uniform int isWater; 

in vec4 FragPosLightSpace;

// variante 2
in vec4 ShadowCoord;

in vec2 UVs;
in vec3 position_worldspace; // position of the vertex (and fragment) in world space
in vec3 normal_cameraspace;
in vec3 eyeDirection_cameraspace; // view direction
in vec3 varyingNormalDirection; 

out vec4 fragColor;

vec4 scene_ambient = vec4(0.3, 0.3, 0.3, 1.0);
//vec4 scene_ambient = vec4(0.5, 0.5, 0.5, 1.0);
float GAMMA = 1;


void main()
{

	vec3 color = vec3(1,1,1);
	float lightIntensity = 30.0f;
	float sharpness = 0;
	vec3 MaterialDiffuseColor = texture(diffuseTexture, UVs).rgb;
	vec3 MaterialAmbientColor = vec3(0.1,0.1,0.1) * MaterialDiffuseColor;
	vec3 MaterialSpecularColor =  vec3(1.0,1.0,1.0);
	
	vec3 normalDirection = normalize(varyingNormalDirection);
	vec3 viewDirection = normalize(eyeDirection_cameraspace);
	
	// calculate visibility for shadows
	float visibility = textureProj(shadowMap, ShadowCoord);
	
	vec3 totalLighting = (vec3(scene_ambient)  * MaterialDiffuseColor);
	vec3 n = normalize(varyingNormalDirection); // Normal of the computed fragment, in camera space, normal direction
	//vec3 viewDirection = normalize(eyeDirection_cameraspace); 	// Eye vector (towards the camera)
	

	float distance; 
	vec3 lightDirection; 
	float attenuation;
	//n = vec3(1, 1, 1);
	// TODO improve function
	for (int i=0; i < lightPointsLen; i++){
		distance = length( pointLights[i].Position - position_worldspace ); // distance to light
		attenuation = 1.0 / (pointLights[i].constantAttenuation + (pointLights[i].linearAttenuation 
			* distance) + (pointLights[i].quadraticAttenuation * distance * distance));
		//lightDirection = (pointLights[i].Position); // for directionallight TODO
		lightDirection = vec3( position_worldspace - pointLights[i].Position);
		
		vec3 l = normalize( lightDirection );  // Direction of the light (from the fragment to the light), lightDirection
		

		// spotlight 
		if (pointLights[i].spotCutoff <= 90.0){
			   // spotlight?
			  float clampedCosine = max(0.0, dot(-l, normalize(pointLights[i].spotDirection) * -1));
			  if (clampedCosine < cos(radians(pointLights[i].spotCutoff))){
				// outside of spotlight cone?
				attenuation = 0.0;
			}
			 else{
				 attenuation = attenuation * pow(clampedCosine, pointLights[i].spotExponent);
			}
		}
		

		// calculate diffuse color
		 vec3 diffuseReflection = attenuation
		* vec3(pointLights[i].diffuse) * MaterialDiffuseColor 
		* max(0.0, dot(n, l)) ;
		// TODO: set specular, 
		vec3 specularReflection; 
		if (dot(n, l) < 0.0){
			// light source on the wrong side?
			specularReflection = vec3(0.0, 0.0, 0.0); // no specular reflection
		
		}else{
			// We dont need specular light
			 // light source on the right side
			 // setSpecular
			//specularReflection	= attenuation * vec3(pointLights[i].specular) * vec3(MaterialSpecularColor) 
			//	* pow(max(0.0, dot(reflect(-l, n), viewDirection*-1)), 0.5);

		}

		if (i == activeLight && visibility < 1){
			totalLighting += diffuseReflection * (visibility -0.2f) + specularReflection * (visibility - 0.2f);
			//totalLighting =   texture(lightshaftMap, UVs).rgb;
		}
		else{
			totalLighting += diffuseReflection + specularReflection;
		}
	}
	
	fragColor.rgb += (totalLighting);


	/*if (false && isWater == 1){
		vec3 vNormal = normalize(normal_cameraspace); 
		float fresnelTerm = dot(position_worldspace, position_worldspace);
		fresnelTerm = clamp(1.0 - fresnelTerm, 0., 1.);
		fragColor = vec4(vec3(0, 1, 1) * fresnelTerm, 1.);

		//fragColor = vec4(1, 0, 0, 1); 
	}*/

	//fragColor.rgb = pow(fragColor.rgb, vec3(1.0/GAMMA));
	fragColor.a=transparency;
		
}

   